import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { Logger } from '@nestjs/common';

import { PropogateDeleteSpaceModelCommand } from '../commands';
import { SpaceRepository } from '@app/common/modules/space';
import { SpaceService } from '../../space/services/space.service';

@CommandHandler(PropogateDeleteSpaceModelCommand)
export class PropogateDeleteSpaceModelHandler
  implements ICommandHandler<PropogateDeleteSpaceModelCommand>
{
  private readonly logger = new Logger(PropogateDeleteSpaceModelHandler.name);

  constructor(
    private readonly spaceRepository: SpaceRepository,
    private readonly spaceService: SpaceService,
  ) {}

  async execute(command: PropogateDeleteSpaceModelCommand): Promise<void> {
    const { spaceModel, queryRunner } = command.param;

    try {
      await queryRunner.startTransaction();
      const spaces = await this.spaceRepository.find({
        where: {
          spaceModel: {
            uuid: spaceModel.uuid,
          },
        },
        relations: [
          'subspaces',
          'productAllocations',
          'subspaces.productAllocations',
        ],
      });

      for (const space of spaces) {
        try {
          await queryRunner.manager.update(
            this.spaceRepository.target,
            { uuid: space.uuid },
            {
              spaceModel: null,
            },
          );
          await this.spaceService.unlinkSpaceFromModel(space, queryRunner);
        } catch (innerError) {
          this.logger.error(
            `Error unlinking space model for space with UUID ${space.uuid}:`,
            innerError.stack || innerError,
          );
        }
      }
      await queryRunner.commitTransaction();
    } catch (error) {
      await queryRunner.rollbackTransaction();
      this.logger.error(
        'Error propagating delete space model:',
        error.stack || error,
      );
    } finally {
      await queryRunner.release();
    }
  }
}
