import { SpaceLinkEntity } from '@app/common/modules/space/entities/space-link.entity';
import { SpaceEntity } from '@app/common/modules/space/entities/space.entity';
import { SpaceLinkRepository } from '@app/common/modules/space/repositories';
import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { QueryRunner } from 'typeorm';

@Injectable()
export class SpaceLinkService {
  constructor(private readonly spaceLinkRepository: SpaceLinkRepository) {}

  async saveSpaceLink(
    startSpaceId: string,
    endSpaceId: string,
    direction: string,
    queryRunner: QueryRunner,
  ): Promise<void> {
    try {
      // Check if a link between the startSpace and endSpace already exists
      const existingLink = await queryRunner.manager.findOne(SpaceLinkEntity, {
        where: {
          startSpace: { uuid: startSpaceId },
          endSpace: { uuid: endSpaceId },
          disabled: false,
        },
      });

      if (existingLink) {
        // Update the direction if the link exists
        existingLink.direction = direction;
        await queryRunner.manager.save(SpaceLinkEntity, existingLink);
        return;
      }

      const existingEndSpaceLink = await queryRunner.manager.findOne(
        SpaceLinkEntity,
        {
          where: { endSpace: { uuid: endSpaceId } },
        },
      );

      if (
        existingEndSpaceLink &&
        existingEndSpaceLink.startSpace.uuid !== startSpaceId
      ) {
        throw new Error(
          `Space with ID ${endSpaceId} is already an endSpace in another link and cannot be reused.`,
        );
      }

      // Find start space
      const startSpace = await queryRunner.manager.findOne(SpaceEntity, {
        where: { uuid: startSpaceId },
      });

      if (!startSpace) {
        throw new HttpException(
          `Start space with ID ${startSpaceId} not found.`,
          HttpStatus.NOT_FOUND,
        );
      }

      // Find end space
      const endSpace = await queryRunner.manager.findOne(SpaceEntity, {
        where: { uuid: endSpaceId },
      });

      if (!endSpace) {
        throw new HttpException(
          `End space with ID ${endSpaceId} not found.`,
          HttpStatus.NOT_FOUND,
        );
      }

      // Create and save the space link
      const spaceLink = this.spaceLinkRepository.create({
        startSpace,
        endSpace,
        direction,
      });

      await queryRunner.manager.save(SpaceLinkEntity, spaceLink);
    } catch (error) {
      throw new HttpException(
        error.message ||
          `Failed to save space link. Internal Server Error: ${error.message}`,
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async deleteSpaceLink(
    space: SpaceEntity,
    queryRunner: QueryRunner,
  ): Promise<void> {
    try {
      const spaceLinks = await queryRunner.manager.find(SpaceLinkEntity, {
        where: [
          { startSpace: space, disabled: false },
          { endSpace: space, disabled: false },
        ],
      });

      if (spaceLinks.length === 0) {
        return;
      }

      const linkIds = spaceLinks.map((link) => link.uuid);

      await queryRunner.manager
        .createQueryBuilder()
        .update(SpaceLinkEntity)
        .set({ disabled: true })
        .whereInIds(linkIds)
        .execute();
    } catch (error) {
      throw new HttpException(
        `Failed to disable space links for the given space: ${error.message}`,
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
}
