import { SpaceProductAllocationRepository } from '@app/common/modules/space';
import { SubspaceProductAllocationEntity } from '@app/common/modules/space/entities/subspace/subspace-product-allocation.entity';
import { SubspaceEntity } from '@app/common/modules/space/entities/subspace/subspace.entity';
import { SubspaceProductAllocationRepository } from '@app/common/modules/space/repositories/subspace.repository';
import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { UpdateSpaceAllocationDto } from 'src/space/interfaces/update-subspace-allocation.dto';
import { TagService as NewTagService } from 'src/tags/services';
import { In, Not, QueryRunner } from 'typeorm';

@Injectable()
export class SubspaceProductAllocationService {
  constructor(
    private readonly tagService: NewTagService,

    private readonly spaceProductAllocationRepository: SpaceProductAllocationRepository,
    private readonly subspaceProductAllocationRepository: SubspaceProductAllocationRepository,
  ) {}

  async createProductAllocations(
    subspace: SubspaceEntity,
    allocationsData: { product: string; tag: string }[],
    queryRunner?: QueryRunner,
    // spaceAllocationsToExclude?: SpaceProductAllocationEntity[],
  ): Promise<void> {
    try {
      if (!allocationsData.length) return;

      const allocations: SubspaceProductAllocationEntity[] = [];

      for (const allocationData of allocationsData) {
        // await this.validateTagWithinSubspace(
        //   queryRunner,
        //   allocationData.tag,
        //   subspace,
        //   spaceAllocationsToExclude,
        // );

        if (
          await this.isAllocationExist(allocationData, subspace, queryRunner)
        ) {
          continue;
        }

        const allocation = this.createNewSubspaceAllocation(
          subspace,
          allocationData,
          queryRunner,
        );
        allocations.push(allocation);
      }
      if (allocations.length > 0) {
        await this.saveAllocations(allocations, queryRunner);
      }
    } catch (error) {
      throw this.handleError(
        error,
        'Failed to create subspace product allocations',
      );
    }
  }

  async updateSubspaceProductAllocationsV2(
    subSpaces: UpdateSpaceAllocationDto[],
    projectUuid: string,
    queryRunner: QueryRunner,
  ) {
    await Promise.all(
      subSpaces.map(async (subspace) => {
        await queryRunner.manager.delete(SubspaceProductAllocationEntity, {
          subspace: { uuid: subspace.uuid },
          tag: {
            uuid: Not(
              In(
                subspace.tags.filter((tag) => tag.uuid).map((tag) => tag.uuid),
              ),
            ),
          },
        });
        const subspaceEntity = await queryRunner.manager.findOne(
          SubspaceEntity,
          {
            where: { uuid: subspace.uuid },
          },
        );

        const processedTags = await this.tagService.processTags(
          subspace.tags,
          projectUuid,
          queryRunner,
        );

        const createdTagsByUUID = new Map(
          processedTags.map((t) => [t.uuid, t]),
        );
        const createdTagsByName = new Map(
          processedTags.map((t) => [t.name, t]),
        );

        // Create the product-tag mapping based on the processed tags
        const productTagMapping = subspace.tags.map(
          ({ uuid, name, productUuid }) => {
            const inputTag = uuid
              ? createdTagsByUUID.get(uuid)
              : createdTagsByName.get(name);
            return {
              tag: inputTag?.uuid,
              product: productUuid,
            };
          },
        );

        await this.createProductAllocations(
          subspaceEntity,
          productTagMapping,
          queryRunner,
        );
      }),
    );
  }

  // async processDeleteActions(dtos: ModifyTagDto[], queryRunner: QueryRunner) {
  //   // : Promise<SubspaceProductAllocationEntity[]>
  //   try {
  //     // if (!dtos || dtos.length === 0) {
  //     //   throw new Error('No DTOs provided for deletion.');
  //     // }
  //     // const tagUuidsToDelete = dtos
  //     //   .filter((dto) => dto.action === ModifyAction.DELETE && dto.tagUuid)
  //     //   .map((dto) => dto.tagUuid);
  //     // if (tagUuidsToDelete.length === 0) return [];
  //     // const allocationsToUpdate = await queryRunner.manager.find(
  //     //   SubspaceProductAllocationEntity,
  //     //   {
  //     //     where: { tag: In(tagUuidsToDelete) },
  //     //   },
  //     // );
  //     // if (!allocationsToUpdate || allocationsToUpdate.length === 0) return [];
  //     // const deletedAllocations: SubspaceProductAllocationEntity[] = [];
  //     // const allocationUpdates: SubspaceProductAllocationEntity[] = [];
  //     // for (const allocation of allocationsToUpdate) {
  //     //   const updatedTags = allocation.tags.filter(
  //     //     (tag) => !tagUuidsToDelete.includes(tag.uuid),
  //     //   );
  //     //   if (updatedTags.length === allocation.tags.length) {
  //     //     continue;
  //     //   }
  //     //   if (updatedTags.length === 0) {
  //     //     deletedAllocations.push(allocation);
  //     //   } else {
  //     //     allocation.tags = updatedTags;
  //     //     allocationUpdates.push(allocation);
  //     //   }
  //     // }
  //     // if (allocationUpdates.length > 0) {
  //     //   await queryRunner.manager.save(
  //     //     SubspaceProductAllocationEntity,
  //     //     allocationUpdates,
  //     //   );
  //     // }
  //     // if (deletedAllocations.length > 0) {
  //     //   await queryRunner.manager.remove(
  //     //     SubspaceProductAllocationEntity,
  //     //     deletedAllocations,
  //     //   );
  //     // }
  //     // await queryRunner.manager
  //     //   .createQueryBuilder()
  //     //   .delete()
  //     //   .from('subspace_product_tags')
  //     //   .where(
  //     //     'subspace_product_allocation_uuid NOT IN ' +
  //     //       queryRunner.manager
  //     //         .createQueryBuilder()
  //     //         .select('allocation.uuid')
  //     //         .from(SubspaceProductAllocationEntity, 'allocation')
  //     //         .getQuery() +
  //     //       ')',
  //     //   )
  //     //   .execute();
  //     // return deletedAllocations;
  //   } catch (error) {
  //     throw this.handleError(error, `Failed to delete tags in subspace`);
  //   }
  // }

  async unlinkModels(
    allocations: SubspaceProductAllocationEntity[],
    queryRunner: QueryRunner,
  ) {
    try {
      if (allocations.length === 0) return;

      const allocationUuids = allocations.map((allocation) => allocation.uuid);

      await queryRunner.manager.update(
        SubspaceProductAllocationEntity,
        { uuid: In(allocationUuids) },
        { inheritedFromModel: null },
      );
    } catch (error) {
      throw new HttpException(
        'Failed to unlink models',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  // private async validateTagWithinSubspace(
  //   queryRunner: QueryRunner | undefined,
  //   tag: NewTagEntity & { product: string },
  //   subspace: SubspaceEntity,
  //   spaceAllocationsToExclude?: SpaceProductAllocationEntity[],
  // ): Promise<void> {
  //   // const existingTagInSpace = await (queryRunner
  //   //   ? queryRunner.manager.findOne(SpaceProductAllocationEntity, {
  //   //       where: {
  //   //         product: { uuid: tag.product },
  //   //         space: { uuid: subspace.space.uuid },
  //   //         tag: { uuid: tag.uuid },
  //   //       },
  //   //     })
  //   //   : this.spaceProductAllocationRepository.findOne({
  //   //       where: {
  //   //         product: { uuid: tag.product },
  //   //         space: { uuid: subspace.space.uuid },
  //   //         tag: { uuid: tag.uuid },
  //   //       },
  //   //     }));
  //   // const isExcluded = spaceAllocationsToExclude?.some(
  //   //   (excludedAllocation) =>
  //   //     excludedAllocation.product.uuid === tag.product &&
  //   //     excludedAllocation.tags.some((t) => t.uuid === tag.uuid),
  //   // );
  //   // if (!isExcluded && existingTagInSpace) {
  //   //   throw new HttpException(
  //   //     `Tag ${tag.uuid} (Product: ${tag.product}) is already allocated at the space level (${subspace.space.uuid}). Cannot allocate the same tag in a subspace.`,
  //   //     HttpStatus.BAD_REQUEST,
  //   //   );
  //   // }
  //   // // ?: Check if the tag is already allocated in another "subspace" within the same space
  //   // const existingTagInSameSpace = await (queryRunner
  //   //   ? queryRunner.manager.findOne(SubspaceProductAllocationEntity, {
  //   //       where: {
  //   //         product: { uuid: tag.product },
  //   //         subspace: { space: subspace.space },
  //   //         tag: { uuid: tag.uuid },
  //   //       },
  //   //       relations: ['subspace'],
  //   //     })
  //   //   : this.subspaceProductAllocationRepository.findOne({
  //   //       where: {
  //   //         product: { uuid: tag.product },
  //   //         subspace: { space: subspace.space },
  //   //         tag: { uuid: tag.uuid },
  //   //       },
  //   //       relations: ['subspace'],
  //   //     }));
  //   // if (
  //   //   existingTagInSameSpace &&
  //   //   existingTagInSameSpace.subspace.uuid !== subspace.uuid
  //   // ) {
  //   //   throw new HttpException(
  //   //     `Tag ${tag.uuid} (Product: ${tag.product}) is already allocated in another subspace (${existingTagInSameSpace.subspace.uuid}) within the same space (${subspace.space.uuid}).`,
  //   //     HttpStatus.BAD_REQUEST,
  //   //   );
  //   // }
  // }

  private createNewSubspaceAllocation(
    subspace: SubspaceEntity,
    allocationData: { product: string; tag: string },
    queryRunner?: QueryRunner,
  ): SubspaceProductAllocationEntity {
    return queryRunner
      ? queryRunner.manager.create(SubspaceProductAllocationEntity, {
          subspace,
          product: { uuid: allocationData.product },
          tag: { uuid: allocationData.tag },
        })
      : this.subspaceProductAllocationRepository.create({
          subspace,
          product: { uuid: allocationData.product },
          tag: { uuid: allocationData.tag },
        });
  }

  private async isAllocationExist(
    allocationData: { product: string; tag: string },
    subspace: SubspaceEntity,
    queryRunner?: QueryRunner,
  ): Promise<boolean> {
    const allocation = queryRunner
      ? await queryRunner.manager.findOne(SubspaceProductAllocationEntity, {
          where: {
            subspace: { uuid: subspace.uuid },
            product: { uuid: allocationData.product },
            tag: { uuid: allocationData.tag },
          },
        })
      : await this.subspaceProductAllocationRepository.findOne({
          where: {
            subspace: { uuid: subspace.uuid },
            product: { uuid: allocationData.product },
            tag: { uuid: allocationData.tag },
          },
        });
    return !!allocation;
  }
  private async saveAllocation(
    allocation: SubspaceProductAllocationEntity,
    queryRunner?: QueryRunner,
  ): Promise<void> {
    if (queryRunner) {
      await queryRunner.manager.save(
        SubspaceProductAllocationEntity,
        allocation,
      );
    } else {
      await this.subspaceProductAllocationRepository.save(allocation);
    }
  }

  private async saveAllocations(
    allocations: SubspaceProductAllocationEntity[],
    queryRunner?: QueryRunner,
  ): Promise<void> {
    if (queryRunner) {
      await queryRunner.manager.save(
        SubspaceProductAllocationEntity,
        allocations,
      );
    } else {
      await this.subspaceProductAllocationRepository.save(allocations);
    }
  }
  private handleError(error: any, message: string): HttpException {
    return new HttpException(
      error instanceof HttpException ? error.message : message,
      error instanceof HttpException
        ? error.getStatus()
        : HttpStatus.INTERNAL_SERVER_ERROR,
    );
  }
  async clearAllAllocations(subspaceUuids: string[], queryRunner: QueryRunner) {
    try {
      await queryRunner.manager.delete(SubspaceProductAllocationEntity, {
        subspace: { uuid: In(subspaceUuids) },
      });
    } catch (error) {
      throw new HttpException(
        error instanceof HttpException
          ? error.message
          : 'An unexpected error occurred while clearing allocations',
        error instanceof HttpException
          ? error.getStatus()
          : HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
}
