import { BaseResponseDto } from '@app/common/dto/base.response.dto';
import { ProjectParam } from '@app/common/dto/project-param.dto';
import { SuccessResponseDto } from '@app/common/dto/success.response.dto';
import { ProductEntity } from '@app/common/modules/product/entities';
import { ProductRepository } from '@app/common/modules/product/repositories';
import { ProjectEntity } from '@app/common/modules/project/entities';
import { ProjectRepository } from '@app/common/modules/project/repositiories';
import { NewTagEntity } from '@app/common/modules/tag';
import { NewTagRepository } from '@app/common/modules/tag/repositories/tag-repository';
import {
  ConflictException,
  HttpException,
  HttpStatus,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { In, QueryRunner } from 'typeorm';
import { ProcessTagDto } from '../dtos';
import { CreateTagDto } from '../dtos/tags.dto';

@Injectable()
export class TagService {
  constructor(
    private readonly tagRepository: NewTagRepository,
    private readonly productRepository: ProductRepository,
    private readonly projectRepository: ProjectRepository,
  ) {}

  async getTagsByProjectUuid(params: ProjectParam): Promise<BaseResponseDto> {
    const { projectUuid } = params;
    await this.getProjectByUuid(projectUuid);

    const tags = await this.tagRepository.find({
      where: { project: { uuid: projectUuid } },
      relations: ['project'],
    });

    return new SuccessResponseDto({
      message: `Tags retrieved successfully for project UUID: ${projectUuid}`,
      data: tags,
    });
  }

  async createTag(
    dto: CreateTagDto,
    param: ProjectParam,
  ): Promise<BaseResponseDto> {
    const { name, productUuid } = dto;

    const product = await this.getProductByUuid(productUuid);
    const project = await this.getProjectByUuid(param.projectUuid);

    await this.validateTagUniqueness(name, param.projectUuid);

    const tag = this.tagRepository.create({
      name,
      product,
      project,
    } as Partial<NewTagEntity>);

    await this.tagRepository.save(tag);

    return new SuccessResponseDto({
      message: `Tag created successfully`,
      data: tag,
    });
  }

  /**
   * Processes an array of tag DTOs, creating or updating tags in the database.
   * @param tagDtos - The array of tag DTOs to process.
   * @param projectUuid - The UUID of the project to associate the tags with.
   * @param queryRunner - Optional TypeORM query runner for transaction management.
   * @returns An array of the processed tag entities.
   */
  async processTags(
    tagDtos: ProcessTagDto[],
    projectUuid: string,
    queryRunner?: QueryRunner,
  ): Promise<NewTagEntity[]> {
    try {
      const dbManager = queryRunner
        ? queryRunner.manager
        : this.tagRepository.manager;
      if (!tagDtos || tagDtos.length === 0) {
        return [];
      }

      const [tagsWithUuid, tagsWithoutUuid]: [
        Pick<ProcessTagDto, 'uuid' | 'productUuid'>[],
        Omit<ProcessTagDto, 'uuid'>[],
      ] = this.splitTagsByUuid(tagDtos);

      // create a set of unique existing tag names for the project
      const upsertedTagsByNameResult = await dbManager.upsert(
        NewTagEntity,
        Array.from(
          new Set<string>(tagsWithoutUuid.map((tag) => tag.name)).values(),
        ).map((name) => ({
          name,
          project: { uuid: projectUuid },
        })),
        ['name', 'project'],
      );

      const createdTagsByName = await dbManager.find(NewTagEntity, {
        where: {
          uuid: In(upsertedTagsByNameResult.identifiers.map((id) => id.uuid)),
        },
      });

      let foundByUuidTags: NewTagEntity[] = [];
      // Fetch existing tags using UUIDs
      if (tagsWithUuid.length) {
        foundByUuidTags = await dbManager.find(NewTagEntity, {
          where: {
            uuid: In([...tagsWithUuid.map((tag) => tag.uuid)]),
            project: { uuid: projectUuid },
          },
        });
      }

      // Ensure all provided UUIDs exist in the database
      if (foundByUuidTags.length !== tagsWithUuid.length) {
        const foundUuids = new Set(foundByUuidTags.map((tag) => tag.uuid));
        const missingUuids = tagsWithUuid.filter(
          ({ uuid }) => !foundUuids.has(uuid),
        );

        throw new HttpException(
          `Some provided tag UUIDs do not exist in the project: ${missingUuids.join(', ')}`,
          HttpStatus.NOT_FOUND,
        );
      }

      return [...foundByUuidTags, ...createdTagsByName];
    } catch (error) {
      console.error(error);
      throw new HttpException(
        error instanceof HttpException
          ? error.message
          : 'An unexpected error occurred while processing tags',
        error instanceof HttpException
          ? error.getStatus()
          : HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  private async getProductByUuid(uuid: string): Promise<ProductEntity> {
    const product = await this.productRepository.findOne({ where: { uuid } });
    if (!product) {
      throw new NotFoundException(`Product with UUID ${uuid} not found.`);
    }
    return product;
  }

  private async getProjectByUuid(uuid: string): Promise<ProjectEntity> {
    const project = await this.projectRepository.findOne({ where: { uuid } });
    if (!project) {
      throw new NotFoundException(`Project with UUID ${uuid} not found.`);
    }
    return project;
  }

  private async validateTagUniqueness(
    name: string,
    projectUuid: string,
  ): Promise<void> {
    const existingTag = await this.tagRepository.findOne({
      where: { name, project: { uuid: projectUuid } },
    });

    if (existingTag) {
      throw new ConflictException(
        `Tag with name "${name}" already exists in this project.`,
      );
    }
  }

  private splitTagsByUuid(
    tagDtos: ProcessTagDto[],
  ): [ProcessTagDto[], ProcessTagDto[]] {
    return tagDtos.reduce<[ProcessTagDto[], ProcessTagDto[]]>(
      ([withUuid, withoutUuid], tag) => {
        if (tag.uuid) {
          withUuid.push(tag);
        } else {
          if (!tag.name || !tag.productUuid) {
            throw new HttpException(
              `Tag name or product UUID is missing`,
              HttpStatus.BAD_REQUEST,
            );
          }
          withoutUuid.push(tag);
        }
        return [withUuid, withoutUuid];
      },
      [[], []],
    );
  }
}
